@@ -1,21 +1,19 @@
 import calendar
-from collections import defaultdict
-from datetime import datetime, timedelta
-from time import sleep
-from time import time
-from hashlib import sha256
 import json
 import logging
+from collections import defaultdict
+from datetime import datetime, timedelta
+from hashlib import sha256
+from time import sleep, time
 
+import httplib2
+import six
 from apiclient.discovery import build
 from apiclient.errors import HttpError
-import httplib2
 
+from bigquery.errors import (BigQueryTimeoutException, JobExecutingException,
+                             JobInsertException, UnfinishedQueryException)
 from bigquery.schema_builder import schema_from_record
-from bigquery.errors import (
-    JobExecutingException, JobInsertException,
-    UnfinishedQueryException, BigQueryTimeoutException
-)
 
 BIGQUERY_SCOPE = 'https://www.googleapis.com/auth/bigquery'
 BIGQUERY_SCOPE_READ_ONLY = 'https://www.googleapis.com/auth/bigquery.readonly'
@@ -154,7 +152,7 @@ class BigQueryClient(object):
                 projectId=self.project_id, body=query_data).execute()
         except HttpError as e:
             if query_data.get("dryRun", False):
-                return None, json.loads(e.content)
+                return None, json.loads(e.content.decode('utf8'))
             raise
 
         job_id = query_reply['jobReference'].get('jobId')
@@ -266,7 +264,7 @@ class BigQueryClient(object):
                 projectId=self.project_id,
                 tableId=table,
                 datasetId=dataset).execute()
-        except HttpError, e:
+        except HttpError as e:
             if int(e.resp['status']) == 404:
                 logging.warn('Table %s.%s does not exist', dataset, table)
                 return None
@@ -651,7 +649,7 @@ class BigQueryClient(object):
                               skip_leading_rows=skip_leading_rows,
                               quote=quote)
             non_null_values = dict((k, v) for k, v
-                                   in all_values.items()
+                                   in list(all_values.items())
                                    if v)
             raise Exception("Parameters field_delimiter, allow_jagged_rows, "
                             "allow_quoted_newlines, quote and "
@@ -837,6 +835,7 @@ class BigQueryClient(object):
         Waits until the job indicated by job_resource is done or has failed
         Args:
             job: dict, representing a BigQuery job resource
+                 or str, representing a BigQuery job id
             interval: optional float polling interval in seconds, default = 5
             timeout: optional float timeout in seconds, default = 60
         Returns:
@@ -848,7 +847,9 @@ class BigQueryClient(object):
             BigQueryTimeoutException on timeout
         """
         complete = False
-        job_id = job['jobReference']['jobId']
+        job_id = str(job if isinstance(job,
+                                       (six.binary_type, six.text_type, int))
+                     else job['jobReference']['jobId'])
         job_resource = None
 
         start_time = time()
@@ -1048,7 +1049,7 @@ class BigQueryClient(object):
             A list of table names that are inside the time range.
         """
 
-        return [table_name for (table_name, unix_seconds) in tables.iteritems()
+        return [table_name for (table_name, unix_seconds) in tables.items()
                 if self._in_range(start_time, end_time, unix_seconds)]
 
     def _in_range(self, start_time, end_time, time):
@@ -1167,7 +1168,7 @@ class BigQueryClient(object):
         Returns:
             string of hexed uris
         """
-        return sha256(":".join(uris) + str(time())).hexdigest()
+        return sha256((":".join(uris) + str(time())).encode()).hexdigest()
 
     def _raise_insert_exception_if_error(self, job):
         error_http = job.get('error')
